@@ -11,11 +11,12 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-
+import collections
 import datetime
 import json
 import os
 import platform
+import re
 import sqlite3
 import time
 from typing import Optional, Type
@@ -24,6 +25,7 @@ import click
 import colorama
 import requests
 from colorama import Fore, Style
+from lupa import LuaRuntime
 
 from tibiawikisql import Image, WikiClient, __version__, models, schema
 from tibiawikisql.models import abc
@@ -34,6 +36,8 @@ DATABASE_FILE = "tibiawiki.db"
 
 colorama.init()
 
+lua = LuaRuntime()
+
 
 def progress_bar(iterable, label, length, **kwargs):
     return click.progressbar(iterable=iterable, length=length, label=label, fill_char="█", empty_char="░", width=10,
@@ -53,23 +57,24 @@ class Category:
 
     Class for internal use only, for easier autocompletion and maintenance."""
     def __init__(self, name: Optional[str], model: Type[abc.Row] = None, *, no_images=False, extension=".gif",
-                 include_deprecated=False, no_title=False):
+                 include_deprecated=False, no_title=False, generate_map=False):
         self.name = name
         self.model = model
         self.no_images = no_images
         self.extension = extension
         self.include_deprecated = include_deprecated
         self.no_title = no_title
+        self.generate_map = generate_map
 
 
 categories = {
     "achievements": Category("Achievements", models.Achievement, no_images=True),
-    "spells": Category("Spells", models.Spell),
-    "items": Category("Objects", models.Item),
-    "creatures": Category("Creatures", models.Creature),
+    "spells": Category("Spells", models.Spell, generate_map=True),
+    "items": Category("Objects", models.Item, generate_map=True),
+    "creatures": Category("Creatures", models.Creature, generate_map=True),
     "books": Category("Book Texts", models.Book, no_images=True),
     "keys": Category("Keys", models.Key, no_images=True),
-    "npcs": Category("NPCs", models.Npc),
+    "npcs": Category("NPCs", models.Npc, generate_map=True),
     "imbuements": Category("Imbuements", models.Imbuement, extension=".png"),
     "quests": Category("Quest Overview Pages", models.Quest, no_images=True),
     "house": Category("Player-Ownable Buildings", models.House, no_images=True),
@@ -111,6 +116,9 @@ def generate(skip_images, db_name, skip_deprecated):
         if not issubclass(model, abc.Parseable):
             continue
         titles = [a.title for a in data_store[key]]
+
+        if value.generate_map:
+            data_store[f"{key}_map"] = {}
         unparsed = []
         exec_time = time.perf_counter()
         generator = WikiClient.get_articles(titles)
@@ -120,6 +128,8 @@ def generate(skip_images, db_name, skip_deprecated):
                     entry = model.from_article(article)
                     if entry is not None:
                         entry.insert(conn)
+                        if value.generate_map:
+                            data_store[f"{key}_map"][entry.title.lower()] = entry.article_id
                     else:
                         unparsed.append(article.title)
             if unparsed:
@@ -131,7 +141,9 @@ def generate(skip_images, db_name, skip_deprecated):
     for position in rashid_positions:
         position.insert(conn)
 
-    generate_loot_statistics(conn)
+    generate_item_offers(conn, data_store)
+    generate_spell_offers(conn, data_store)
+    generate_loot_statistics(conn, data_store)
 
     if not skip_images:
         with conn:
@@ -154,45 +166,165 @@ def generate(skip_images, db_name, skip_deprecated):
     click.echo(f"Command finished in {dt:.2f} seconds.")
 
 
-def generate_loot_statistics(conn: sqlite3.Connection):
+link_pattern = re.compile(r"(?:(?P<price>\d+))?\s?\[\[([^\]|]+)")
+
+
+def generate_spell_offers(conn: sqlite3.Connection, data_store):
+    if "npcs_map" not in data_store or "spells_map" not in data_store:
+        return
+    start_time = time.perf_counter()
+    article = WikiClient.get_article("Module:ItemPrices/spelldata")
+    data = lua.execute(article.content)
+
+    spell_offers = []
+    not_found_store = collections.defaultdict(set)
+    spells = list(data.items())
+    with progress_bar(spells, "Fetching spell offers", len(spells)) as bar:
+        for name, table in bar:
+            spell_id = data_store["spells_map"].get(name.lower())
+            if spell_id is None:
+                not_found_store["spell"].add(name)
+                continue
+            spell_vocations = list((table["vocation"].values()))
+            for npc, vocation in table["sellers"].items():
+                npc_id = data_store["npcs_map"].get(npc.lower())
+                if npc_id is None:
+                    not_found_store["item"].add(npc)
+                    continue
+                if isinstance(vocation, bool):
+                    npc_vocations = spell_vocations
+                elif isinstance(vocation, str):
+                    npc_vocations = [vocation]
+                else:
+                    npc_vocations = list(vocation.values())
+                spell_offers.append((
+                    npc_id,
+                    spell_id,
+                    "Knight" in npc_vocations,
+                    "Sorcerer" in npc_vocations,
+                    "Druid" in npc_vocations,
+                    "Paladin" in npc_vocations,
+                ))
+        with conn:
+            conn.execute("DELETE FROM npc_spell")
+            conn.executemany("INSERT INTO npc_spell(npc_id, spell_id, knight, sorcerer, paladin, druid) VALUES(?, ?, ?, ?, ?, ?)", spell_offers)
+        if not_found_store["spell"]:
+            unknonw_spells = not_found_store["spell"]
+            click.echo(f"{Fore.RED}Could not parse offers for {len(unknonw_spells):,} spell.{Style.RESET_ALL}")
+            click.echo(f"\t-> {Fore.RED}{f'{Style.RESET_ALL},{Fore.RED}'.join(unknonw_spells)}{Style.RESET_ALL}")
+        if not_found_store["npc"]:
+            unknown_npcs = not_found_store["npc"]
+            click.echo(f"{Fore.RED}Could not parse offers of {len(unknown_npcs):,} npcs.{Style.RESET_ALL}")
+            click.echo(f"\t-> {Fore.RED}{f'{Style.RESET_ALL},{Fore.RED}'.join(unknown_npcs)}{Style.RESET_ALL}")
+
+
+def generate_item_offers(conn: sqlite3.Connection, data_store):
+    if "npcs_map" not in data_store or "items_map" not in data_store:
+        return
+    start_time = time.perf_counter()
+    article = WikiClient.get_article("Module:ItemPrices/data")
+    data = lua.execute(article.content)
+
+    sell_offers = []
+    buy_offers = []
+    not_found_store = collections.defaultdict(set)
+    npc_offers = list(data.items())
+    with progress_bar(npc_offers, "Fetching NPC offers", len(npc_offers)) as bar:
+        for name, table in bar:
+            npc_id = data_store["npcs_map"].get(name.lower())
+            if npc_id is None:
+                not_found_store["npc"].add(name)
+                continue
+            if "sells" in table:
+                process_offer_list(npc_id, table["sells"], sell_offers, data_store, not_found_store)
+            if "buys" in table:
+                process_offer_list(npc_id, table["buys"], buy_offers, data_store, not_found_store)
+        with conn:
+            conn.execute("DELETE FROM npc_offer_sell")
+            conn.execute("DELETE FROM npc_offer_buy")
+            conn.executemany("INSERT INTO npc_offer_sell(npc_id, value, item_id, currency_id) VALUES(?, ?, ?, ?)", sell_offers)
+            conn.executemany("INSERT INTO npc_offer_buy(npc_id, value, item_id, currency_id) VALUES(?, ?, ?, ?)", buy_offers)
+            dt = (time.perf_counter() - start_time)
+    total_offers = len(sell_offers) + len(buy_offers)
+    if not_found_store["npc"]:
+        unknonw_npcs = not_found_store["npc"]
+        click.echo(f"{Fore.RED}Could not parse offers for {len(unknonw_npcs):,} npcs.{Style.RESET_ALL}")
+        click.echo(f"\t-> {Fore.RED}{f'{Style.RESET_ALL},{Fore.RED}'.join(unknonw_npcs)}{Style.RESET_ALL}")
+    if not_found_store["item"]:
+        unknonw_items = not_found_store["item"]
+        click.echo(f"{Fore.RED}Could not parse offers for {len(unknonw_items):,} items.{Style.RESET_ALL}")
+        click.echo(f"\t-> {Fore.RED}{f'{Style.RESET_ALL},{Fore.RED}'.join(unknonw_items)}{Style.RESET_ALL}")
+    click.echo(f"{Fore.GREEN}\tSaved {total_offers:,} NPC offers in {dt:.2f} seconds.")
+
+
+def process_offer_list(npc_id, array, list_store, data_store, not_found):
+    for lua_item in array.values():
+        data = dict(lua_item.items())
+        price = data["price"]
+        currency = data.get("currency", "gold coin")
+        if not isinstance(price, int):
+            m = link_pattern.search(price)
+            price = int(m.group("price") or 1)
+            currency = m.group(2)
+        item_name = data["item"]
+        currency_name = currency
+        item_id = data_store["items_map"].get(item_name.lower())
+        currency_id = data_store["items_map"].get(currency_name.lower())
+        if currency_id is None:
+            not_found["item"].add(currency_name)
+            continue
+        if item_id is None:
+            not_found["item"].add(item_name)
+            continue
+        list_store.append((
+            npc_id,
+            price,
+            item_id,
+            currency_id
+        ))
+
+
+def generate_loot_statistics(conn: sqlite3.Connection, data_store):
     c = conn.cursor()
     try:
         results = conn.execute("SELECT title FROM creature")
         titles = [f"Loot Statistics:{t[0]}" for t in results]
         start_time = time.perf_counter()
+        unknown_items = set()
         with progress_bar(WikiClient.get_articles(titles), "Fetching loot statistics", len(titles),
                           item_show_func=article_show) as bar:
             for article in bar:
                 if article is None:
                     continue
                 creature_title = article.title.replace("Loot Statistics:", "")
-                c.execute("SELECT article_id from creature WHERE title = ?", (creature_title,))
-                result = c.fetchone()
-                if result is None:
+                creature_id = data_store["creatures_map"].get(creature_title.lower())
+                if creature_id is None:
                     # This could happen if a creature's article was deleted but its Loot Statistics weren't
                     continue
-                creature_id = result[0]
-                # Most loot statistics contain stats for older versions too, we onl care about the latest version.
+                # Most loot statistics contain stats for older versions too, we only care about the latest version.
                 kills, loot_stats = parse_loot_statistics(article.content)
                 loot_items = []
                 for entry in loot_stats:
-                    item = entry["item"]
-                    times = entry["times"]
-                    amount = entry.get("amount", 1)
-                    c.execute("SELECT article_id FROM item WHERE title LIKE ?", (item,))
-                    result = c.fetchone()
-                    if result is None:
-                        continue
-                    item_id = result[0]
-                    percentage = min(int(times) / kills * 100, 100)
-                    _min, _max = parse_min_max(amount)
-                    loot_items.append((creature_id, item_id, percentage, _min, _max))
-                    # We delete any duplicate record that was added from the creature's article's loot if it exists
-                    c.execute("DELETE FROM creature_drop WHERE creature_id = ? AND item_id = ?",
-                              (creature_id, item_id))
+                    if entry:
+                        item = entry["item"]
+                        times = entry["times"]
+                        amount = entry.get("amount", 1)
+                        item_id = data_store["items_map"].get(item.lower())
+                        if item_id is None:
+                            unknown_items.add(item)
+                            continue
+                        percentage = min(int(times) / kills * 100, 100)
+                        _min, _max = parse_min_max(amount)
+                        loot_items.append((creature_id, item_id, percentage, _min, _max))
+                        # We delete any duplicate record that was added from the creature's article's loot if it exists
+                        c.execute("DELETE FROM creature_drop WHERE creature_id = ? AND item_id = ?",
+                                (creature_id, item_id))
                 c.executemany("INSERT INTO creature_drop(creature_id, item_id, chance, min, max) VALUES(?,?,?,?,?)",
                               loot_items)
         dt = (time.perf_counter() - start_time)
+        if unknown_items:
+            click.echo(f"{Fore.RED}Could not find {len(unknown_items):,} items.{Style.RESET_ALL}")
+            click.echo(f"\t-> {Fore.RED}{f'{Style.RESET_ALL},{Fore.RED}'.join(unknown_items)}{Style.RESET_ALL}")
         click.echo(f"{Fore.GREEN}\tParsed loot statistics in {dt:.2f} seconds.{Style.RESET_ALL}")
     finally:
         conn.commit()
