@@ -63,6 +63,7 @@ class Creature(Table):
     bestiary_class = Column(Text, index=True)
     bestiary_level = Column(Text, index=True)
     bestiary_occurrence = Column(Text, index=True)
+    bosstiary_class = Column(Text, index=True)
     runs_at = Column(Integer)
     summon_cost = Column(Integer)
     convince_cost = Column(Integer)
@@ -220,6 +221,7 @@ class Imbuement(Table):
     name = Column(Text, unique=True, index=True)
     tier = Column(Text, nullable=False)
     type = Column(Text, nullable=False)
+    category = Column(Text, nullable=False)
     effect = Column(Text)
     slots = Column(Text)
     version = Column(Text, index=True)
@@ -290,6 +292,7 @@ class Npc(Table):
     name = Column(Text, no_case=True, index=True)
     gender = Column(Text, index=True)
     city = Column(Text, index=True)
+    subarea = Column(Text)
     location = Column(Text)
     version = Column(Text, index=True)
     x = Column(Integer)
